#**************************************************************************** 
#** 
#**  File     :  /cdimage/units/URS0201/URS0201_script.lua 
#**  Author(s):  David Tomandl, Jessica St. Croix 
#** 
#**  Summary  :  Cybran Destroyer Script 
#** 
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved. 
#**************************************************************************** 

local CWalkingLandUnit = import('/lua/cybranunits.lua').CWalkingLandUnit 
local CSeaUnit = import('/lua/cybranunits.lua').CSeaUnit 
local CybranWeapons = import('/lua/cybranweapons.lua') 
local CAAAutocannon = CybranWeapons.CAAAutocannon 
local CDFProtonCannonWeapon = CybranWeapons.CDFProtonCannonWeapon 
local CANNaniteTorpedoWeapon = import('/lua/cybranweapons.lua').CANNaniteTorpedoWeapon 
local CIFSmartCharge = import('/lua/cybranweapons.lua').CIFSmartCharge 
local AIUtils = import('/lua/ai/aiutilities.lua') 

URS0201 = Class(CSeaUnit) { 
    SwitchAnims = true, 
    Walking = false, 
    IsWaiting = false, 

    Weapons = { 
        ParticleGun = Class(CDFProtonCannonWeapon) {}, 
        AAGun = Class(CAAAutocannon) {}, 
        TorpedoR = Class(CANNaniteTorpedoWeapon) {}, 
        TorpedoL = Class(CANNaniteTorpedoWeapon) {}, 
        AntiTorpedoF = Class(CIFSmartCharge) {}, 
        AntiTorpedoB = Class(CIFSmartCharge) {}, 
    }, 

    OnCreate = function(self) 
        CSeaUnit.OnCreate(self) 
        #self:HideBone('Turret_Mount_Front', true) 
        #self:HideBone('Turret_Mount_Back', true) 
    end, 

    OnStopBeingBuilt = function(self,builder,layer) 
        CSeaUnit.OnStopBeingBuilt(self,builder,layer) 
        # If created with F2 on land, then play the transform anim. 
        if(self:GetCurrentLayer() == 'Land') then 
            self.AT1 = self:ForkThread(self.TransformThread, true) 
        end 
      self:ForkThread(self.TransportThread) 
    end, 

    OnMotionHorzEventChange = function(self, new, old) 
        CSeaUnit.OnMotionHorzEventChange(self, new, old) 
        if self:IsDead() then return end 
        if( not self.IsWaiting ) then 
            if( self.Walking ) then 
                if( old == 'Stopped' ) then 
                    if( self.SwitchAnims ) then 
                        self.SwitchAnims = false 
                        self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationWalk, true):SetRate(self:GetBlueprint().Display.AnimationWalkRate or 1.1) 
                    else 
                        self.AnimManip:SetRate(2.8) 
                    end 
                elseif( new == 'Stopped' ) then 
                    self.AnimManip:SetRate(0) 
                end 
            end 
        end 
    end, 

    OnLayerChange = function(self, new, old) 
        CSeaUnit.OnLayerChange(self, new, old) 
        if( old != 'None' ) then 
            if( self.AT1 ) then 
                self.AT1:Destroy() 
                self.AT1 = nil 
            end 
            local myBlueprint = self:GetBlueprint() 
            if( new == 'Land' ) then 
                self.AT1 = self:ForkThread(self.TransformThread, true) 
            elseif( new == 'Water' ) then 
                self.AT1 = self:ForkThread(self.TransformThread, false) 
            end 
        end 
    end, 

    TransformThread = function(self, land) 
        if( not self.AnimManip ) then 
            self.AnimManip = CreateAnimator(self) 
        end 
        local bp = self:GetBlueprint() 
        local scale = bp.Display.UniformScale or 1 

        if( land ) then 
            # Change movement speed to the multiplier in blueprint 
            self:SetSpeedMult(bp.Physics.LandSpeedMultiplier) 
            self:SetImmobile(true) 
            self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationTransform) 
            self.AnimManip:SetRate(2) 
            self.IsWaiting = true 
            WaitFor(self.AnimManip) 
            self:SetCollisionShape( 'Box', bp.CollisionOffsetX or 0,(bp.CollisionOffsetY + (bp.SizeY*1.0)) or 0,bp.CollisionOffsetZ or 0, bp.SizeX * scale, bp.SizeY * scale, bp.SizeZ * scale ) 
            self.IsWaiting = false 
            self:SetImmobile(false) 
            self.SwitchAnims = true 
            self.Walking = true 
            self.Trash:Add(self.AnimManip) 
        else 
            self:SetImmobile(true) 
            # Revert speed to maximum speed 
            self:SetSpeedMult(1) 
            self.AnimManip:PlayAnim(self:GetBlueprint().Display.AnimationTransform) 
            self.AnimManip:SetAnimationFraction(1) 
            self.AnimManip:SetRate(-2) 
            self.IsWaiting = true 
            WaitFor(self.AnimManip) 
            self:SetCollisionShape( 'Box', bp.CollisionOffsetX or 0,(bp.CollisionOffsetY + (bp.SizeY * 0.5)) or 0,bp.CollisionOffsetZ or 0, bp.SizeX * scale, bp.SizeY * scale, bp.SizeZ * scale ) 
            self.IsWaiting = false 
            self.AnimManip:Destroy() 
            self.AnimManip = nil 
            self:SetImmobile(false) 
            self.Walking = false 
        end 
    end, 

    OnKilled = function(self, instigator, type, overkillRatio) 
        self.Trash:Destroy() 
        self.Trash = TrashBag() 
        if(self:GetCurrentLayer() != 'Water') then 
            self:GetBlueprint().Display.AnimationDeath = self:GetBlueprint().Display.LandAnimationDeath 
        else 
            self:GetBlueprint().Display.AnimationDeath = self:GetBlueprint().Display.WaterAnimationDeath 
        end 
        CSeaUnit.OnKilled(self, instigator, type, overkillRatio) 
    end, 
    
     DeathThread = function(self, overkillRatio) 
        if (self:GetCurrentLayer() != 'Water') then 
            self:PlayUnitSound('Destroyed') 
            local army = self:GetArmy() 
            if self.PlayDestructionEffects then 
                self:CreateDestructionEffects( self, overkillRatio ) 
            end 

            # Create Initial explosion effects 
            if( self.ShowUnitDestructionDebris and overkillRatio ) then 
                if overkillRatio <= 1 then 
                    self.CreateUnitDestructionDebris( self, true, true, false ) 
                elseif overkillRatio <= 2 then 
                    self.CreateUnitDestructionDebris( self, true, true, false ) 
                elseif overkillRatio <= 3 then 
                    self.CreateUnitDestructionDebris( self, true, true, true ) 
                else #VAPORIZED 
                    self.CreateUnitDestructionDebris( self, true, true, true ) 
                end 
            end 

            WaitSeconds(2) 
            if self.PlayDestructionEffects then 
                self:CreateDestructionEffects( self, overkillRatio ) 
            end 
            WaitSeconds(1) 
            if self.PlayDestructionEffects then 
                self:CreateDestructionEffects( self, overkillRatio ) 
            end                                    
            self:CreateWreckage(0.1) 
            self:Destroy() 
        else 
            CSeaUnit.DeathThread(self, overkillRatio) 
        end 
    end, 

   RemoveUnitFromStorage = function(self, unit) 
      LOG('RemoveUnitFromStorage started') 
      local cargo = self:GetCargo() 
      for k,v in cargo do 
         if v == unit then 
            LOG('found unit') 
            v:OnRemoveFromStorage(self) 
            v:DetachFrom(true) 
         end 
      end 
   end, 
    
    TransportThread = function(self) 
      LOG('TransportThread started') 
         while not self:IsDead() do 
            LOG('I got the alive check just fine') 
            #Get friendly units in the area (including self) 
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.LAND, self:GetPosition(), 20) 
            LOG('Got list of units too') 
            for id,unit in units do 
            local bp = unit:GetBlueprint() 
               LOG('Retrieved the targets blueprint') 
               if unit != self then 
                  LOG('Made sure it wasnt me') 
                  if EntityCategoryContains( categories.EXPERIMENTAL, unit ) then                  
                  else 
                     LOG('Or an Experimental') 
                     if EntityCategoryContains( categories.MOBILE, unit ) then 
                         LOG('Or a structure') 
                         if unit:GetCurrentLayer() != 'Seabed' then 
                              LOG('Or on the sea bed') 
                              if self:TransportHasAvailableStorage() then 
                                 LOG('Lets have him!') 
                                 self:AddUnitToStorage(unit) 
                                 WaitSeconds(5)
                                 if self:GetCurrentLayer() == 'Water' then
                                 else
                                    WaitSeconds(5)
                                    self:RemoveUnitFromStorage(unit)
                                    LOG('My layer was..', self:GetCurrentLayer())
                                 end
  
                            else 
                                 LOG('No room for the poor unit') 
                                 LOG('My blueprint is..', repr(self:GetBlueprint())) 
                              end    
                          end 
                     end 
                  end 
               end 
            end 
        self:RequestRefreshUI() 
        WaitSeconds(15) 
        end 
  
        LOG('TransportThread ran successfully!')    
        
    end, 
} 

TypeClass = URS0201 